import { AliasParams, DispatchedEvent, EventParams, GroupParams, PageParams, IdentifyParams } from '../arguments-resolver';
import type { FormArgs, LinkArgs } from '../auto-track';
import { Context } from '../context';
import { Emitter } from '@segment/analytics-generic-utils';
import { SegmentEvent } from '../events';
import { Plugin } from '../plugin';
import { EventQueue } from '../queue/event-queue';
import { Group, ID, User } from '../user';
import type { LegacyIntegration } from '../../plugins/ajs-destination/types';
import type { DestinationMiddlewareFunction, MiddlewareFunction } from '../../plugins/middleware';
import { AnalyticsClassic, AnalyticsCore } from './interfaces';
import type { CDNSettings } from '../../browser';
import { UniversalStorage } from '../storage';
import { AnalyticsSettings, IntegrationsInitOptions, InitOptions } from '../../browser/settings';
export type { InitOptions, AnalyticsSettings };
/**
 * The public settings that are set on the analytics instance
 */
export declare class AnalyticsInstanceSettings {
    readonly writeKey: string;
    /**
     * This is an unstable API, it may change in the future without warning.
     */
    readonly cdnSettings: CDNSettings;
    readonly cdnURL?: string;
    get apiHost(): string | undefined;
    private _getSegmentPluginMetadata?;
    /**
     * Auto-track specific timeout setting for legacy purposes.
     */
    timeout: number;
    constructor(settings: AnalyticsSettings, queue: EventQueue);
}
declare function _stub(this: never): void;
export declare class Analytics extends Emitter implements AnalyticsCore, AnalyticsClassic {
    settings: AnalyticsInstanceSettings;
    private _user;
    private _group;
    private eventFactory;
    private _debug;
    private _universalStorage;
    initialized: boolean;
    integrations: IntegrationsInitOptions;
    options: InitOptions;
    queue: EventQueue;
    constructor(settings: AnalyticsSettings, options?: InitOptions, queue?: EventQueue, user?: User, group?: Group);
    user: () => User;
    /**
     * Creates the storage system based on the settings received
     * @returns Storage
     */
    private createStore;
    get storage(): UniversalStorage;
    track(...args: EventParams): Promise<DispatchedEvent>;
    page(...args: PageParams): Promise<DispatchedEvent>;
    identify(...args: IdentifyParams): Promise<DispatchedEvent>;
    group(): Group;
    group(...args: GroupParams): Promise<DispatchedEvent>;
    alias(...args: AliasParams): Promise<DispatchedEvent>;
    screen(...args: PageParams): Promise<DispatchedEvent>;
    trackClick(...args: LinkArgs): Promise<Analytics>;
    trackLink(...args: LinkArgs): Promise<Analytics>;
    trackSubmit(...args: FormArgs): Promise<Analytics>;
    trackForm(...args: FormArgs): Promise<Analytics>;
    register(...plugins: Plugin[]): Promise<Context>;
    deregister(...plugins: string[]): Promise<Context>;
    debug(toggle: boolean): Analytics;
    reset(): void;
    timeout(timeout: number): void;
    private _dispatch;
    addSourceMiddleware(fn: MiddlewareFunction): Promise<Analytics>;
    addDestinationMiddleware(integrationName: string, ...middlewares: DestinationMiddlewareFunction[]): Promise<Analytics>;
    setAnonymousId(id?: string): ID;
    queryString(query: string): Promise<Context[]>;
    /**
     * @deprecated This function does not register a destination plugin.
     *
     * Instantiates a legacy Analytics.js destination.
     *
     * This function does not register the destination as an Analytics.JS plugin,
     * all the it does it to invoke the factory function back.
     */
    use(legacyPluginFactory: (analytics: Analytics) => void): Analytics;
    ready(callback?: Function): Promise<unknown>;
    noConflict(): Analytics;
    normalize(msg: SegmentEvent): SegmentEvent;
    get failedInitializations(): string[];
    get VERSION(): string;
    initialize(_settings?: AnalyticsSettings, _options?: InitOptions): Promise<Analytics>;
    init: (_settings?: AnalyticsSettings, _options?: InitOptions) => Promise<Analytics>;
    pageview(url: string): Promise<Analytics>;
    get plugins(): any;
    get Integrations(): Record<string, LegacyIntegration>;
    log: typeof _stub;
    addIntegrationMiddleware: typeof _stub;
    listeners: typeof _stub;
    addEventListener: typeof _stub;
    removeAllListeners: typeof _stub;
    removeListener: typeof _stub;
    removeEventListener: typeof _stub;
    hasListeners: typeof _stub;
    add: typeof _stub;
    addIntegration: typeof _stub;
    push(args: any[]): void;
}
/**
 * @returns a no-op analytics instance that does not create cookies or localstorage, or send any events to segment.
 */
export declare class NullAnalytics extends Analytics {
    constructor();
}
//# sourceMappingURL=index.d.ts.map